#define LED 13
#define RX_PIN 10
#define TX_PIN 11
#define RELAY 2
#include <AltSoftSerial.h>

AltSoftSerial mySerial;

volatile int seconds = 0;
volatile int counter = 0;
volatile bool can_irrigate = false;
volatile int wanted_moisture_percentage = 15;
volatile unsigned long long int last_millis = 0;

void configure_timer2() {
  TCCR2B = 0;
  TCCR2A = 0;
  TCNT2 = 0;
  TCCR2B |= (1 << WGM22) | (1 << CS22) | (1 << CS21) | (1 << CS20);
  TIMSK2 |= (1 << OCIE2A);
  OCR2A = 154;
}

void setup() {
  pinMode(RX_PIN, INPUT);
  pinMode(TX_PIN, OUTPUT);
  pinMode(RELAY, OUTPUT);
  digitalWrite(RELAY, HIGH);
  pinMode(LED, OUTPUT);
  mySerial.begin(9600);
  Serial.begin(9600);
  cli();
  configure_timer2();
  sei();
}

ISR(TIMER2_COMPA_vect) {
    counter++;
    if (counter == 63) {
      seconds++;
      counter = 0;
      Serial.println(seconds);
    }
  if (seconds == 60) {
    can_irrigate = true;
    seconds = 0;
  }
}

void loop() {
  if (can_irrigate) {
    int moisture = analogRead(A0);
    int moisturePercentage = map(moisture, 1023, 250, 0, 100);
    Serial.print("Moisture percentage: ");
    Serial.println(moisturePercentage);
    if (moisturePercentage < wanted_moisture_percentage) {
      // Starting irrigation...
      digitalWrite(RELAY, LOW);
      delay(3000);
      digitalWrite(RELAY, HIGH);
      // Finishing irrigation...
    }
    can_irrigate = false;
  }
  
  if (mySerial.available() > 0)
  {
    char incoming_command = mySerial.read();
    Serial.print(incoming_command);
    if (incoming_command == '1') {
      Serial.println("Turning LEDs on...");
      digitalWrite(LED, HIGH);
    } else if (incoming_command == '0') {
      Serial.println("Turning LEDs off...");
      digitalWrite(LED, LOW);
    } else if (incoming_command == '4') {
      if (wanted_moisture_percentage + 5 <= 100) {
        Serial.println("Incrementing the moisture percentage with 5");
        wanted_moisture_percentage += 5;
        Serial.print("The soil moisture must be under ");
        Serial.print(wanted_moisture_percentage);
        Serial.println(".");
      }
    } else if (incoming_command == '5') {
      if (wanted_moisture_percentage - 5 >= 5) {
        Serial.println("Decrementing the moisture percentage with 5");
        wanted_moisture_percentage -= 5;
        Serial.print("The soil moisture must be under ");
        Serial.print(wanted_moisture_percentage);
        Serial.println(".");
      }
    }
  }
}
